package de.itdesign.meisterplan.shared.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.MappedSuperclass;
import javax.persistence.PostLoad;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.Transient;

@MappedSuperclass
public class BaseModelData implements Serializable {

    private static final long serialVersionUID = -6642595831323468415L;

    public static final String MP_SOURCE_SYSTEM_ID = "MEISTERPLAN"; //$NON-NLS-1$

    @Transient
    private Date unmanagedCreatedDate = null;

    @Transient
    private String unmanagedCreatedBy = null;

    @Transient
    private String unmanagedSourceSystem = null;

    private Date createdDate;
    private String createdBy;

    private Date lastUpdatedDate;
    private String lastUpdatedBy;
    private String sourceSystem;

    /* @see BaseModelData#touch() */
    private long touchHelper = 0L;

    @PostLoad
    protected void postLoad() {
        unmanagedCreatedBy = createdBy;
        unmanagedCreatedDate = createdDate;
        unmanagedSourceSystem = sourceSystem;
    }

    @PrePersist
    protected void prePersist() {
    }

    @PreUpdate
    protected void preUpdate() {
        if (!getClass().getSimpleName().equals("UserSessionModelData")) {
            System.err.println("must not get here");
            throw new IllegalStateException();
            //            int i = 0;
            //            if (i == 3)
            //                return;
        }

    }

    public Date getCreatedDate() {
        return createdDate;
    }

    public void setCreatedDate(Date createdDate) {
        this.createdDate = createdDate;
        this.unmanagedCreatedDate = createdDate;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
        this.unmanagedCreatedBy = createdBy;
    }

    public Date getLastUpdatedDate() {
        return lastUpdatedDate;
    }

    public void setLastUpdatedDate(Date lastUpdatedDate) {
        this.lastUpdatedDate = lastUpdatedDate;
    }

    public String getLastUpdatedBy() {
        return lastUpdatedBy;
    }

    public void setLastUpdatedBy(String lastUpdatedBy) {
        this.lastUpdatedBy = lastUpdatedBy;
    }

    public String getSourceSystem() {
        return sourceSystem;
    }

    public void setSourceSystem(String sourceSystem) {
        this.sourceSystem = sourceSystem;
        this.unmanagedSourceSystem = sourceSystem;
    }

    public boolean isMeisterplanSource() {
        return MP_SOURCE_SYSTEM_ID.equals(sourceSystem);
    }

    /**
     * Method forces JPA connection to invoke preUpdate without any real changes
     * on the object. And later persists the changes.
     */
    public void touch() {
        touchHelper++;
    }

    public void clearBaseData() {
        sourceSystem = null;
        createdBy = null;
        createdDate = null;
        lastUpdatedBy = null;
        lastUpdatedDate = null;
    }

    protected void copyTo(BaseModelData copyTo) {
        copyTo.createdDate = createdDate;
        copyTo.createdBy = createdBy;

        copyTo.lastUpdatedDate = lastUpdatedDate;
        copyTo.lastUpdatedBy = lastUpdatedBy;
        copyTo.sourceSystem = sourceSystem;
    }
}
